const mongoose = require('mongoose');
const express = require('express')
const jimp = require('jimp')
const bp = require('body-parser')
const fs = require('fs')
const app = express()
const port = 80

// DB stuff
const db  = mongoose.connection;
const mongoDBURL = 'mongodb://127.0.0.1/memes';
mongoose.connect(mongoDBURL, { useNewUrlParser: true });
db.on('error', () => { console.log('MongoDB connection error:') });

var MemeSchema = new mongoose.Schema({
  fileName: String,
  text: String
});    
var Meme = mongoose.model('Meme', MemeSchema );
       

app.use(express.static('public_html'))
app.use(bp.json());

app.get('/creatememe/:template/:top/:bottom', function (req, res) {
  var template = req.params.template;
  var topText = req.params.top;
  var bottomText = req.params.bottom;
  var inputFileName = './images/' + template + '.png'
  var imgFile = Math.floor(Math.random() * 10000) + '.png';
  var outputFileName = './public_html/' + imgFile;

 // update to use promises

  let image = undefined;

  jimp.read(inputFileName).then((img) => {
      image = img;
      return jimp.loadFont(jimp.FONT_SANS_32_WHITE);
  }).then((font) => {
      image.print(font, 10, 10, topText);
      image.print(font, 10, 350, bottomText);
      return image.write(outputFileName);
  }).then((image) => {
      let newMeme = new Meme({ fileName: imgFile, text: topText + ' ' + bottomText });
      return newMeme.save();
  }).then((result) => { 
      res.end('/' + imgFile);
  }).catch((err) => { 
      console.log('yikes!');
      console.log(err);
  });

});

function processMemeResults(documents, res) {
  console.log(documents);
  let results = '\n';
  for (let i = 0; i < documents.length; i++) {
    results += '<img class="resultMeme" src="/' + documents[i].fileName + '" />\n';
  }
  console.log(results);
  res.end(results);
}

app.get('/memes/search/:term', function (req, res) {
  console.log('I got here!');
  let p = Meme.find({ text : { $regex : req.params.term } }).exec();
  p.then((documents) => {
    processMemeResults(documents, res);
  });
});

app.get('/memes/get', function (req, res) {
  let p = Meme.find({}).exec();
  p.then((documents) => {
    processMemeResults(documents, res);
  });
});

app.post('/some/route', (req, res) => {
  let a = req.body.name;
  console.log(a);
  res.end('Got the name');
});


app.listen(port, () => 
  console.log(`App listening at http://localhost:${port}`))
